import arcgisscripting
import math
import os

def runRegularPolygons():
    gp = arcgisscripting.create(9.3)
    gp.OverWriteOutput = 1

    # Input point FC
    inPts = gp.GetParameterAsText(0)
    # Output buffer FC
    outRegularPolygon = gp.GetParameterAsText(1)
    # Distance from center to vertices on polygon edge
    distance = float(gp.GetParameterAsText(2))
    # Number of sides
    sidesCnt = int(gp.GetParameterAsText(3))
    # Rotation
    rotation = gp.GetParameterAsText(4)
    if rotation == "":
        rotation = 0
    rotation = float(rotation)
    
        
    createRegularPolygons(gp, inPts, outRegularPolygon, distance, sidesCnt, rotation) 

def createRegularPolygons(gp, inPts, outRegularPolygon, distance, sidesCnt, rotation):
    try:
        # Assign empty values to cursor and row objects
        #
        iCur, sRow, sCur, feat = None, None, None, None

        # Need a field list to move field attributes from input to output
        #
        shapeName = gp.Describe(inPts).ShapeFieldName
        OIDName   = gp.Describe(inPts).OIDFieldName
        fieldList = []
        for field in gp.ListFields(inPts):
            if field.Name not in [shapeName, OIDName]:
                fieldList.append(field.Name)

        # Create the output feature class
        #
        outPath, outFC = os.path.split(outRegularPolygon)
        gp.CreateFeatureClass(outPath, outFC, "Polygon", inPts)
        
        angle = 360.0 / float(sidesCnt)

        # Open an insert cursor for the new feature class
        #
        iCur = gp.InsertCursor(outRegularPolygon)
        sCur = gp.SearchCursor(inPts)
        sRow = sCur.Next()

        # Create an array and point object needed to create features
        #
        lineArray = gp.CreateObject("Array")
        outPoint = gp.CreateObject("Point")

        while sRow:
            # Get point object from input
            #
            inPoint = sRow.GetValue(shapeName).GetPart(0)

            # Create a point object for each vertex in the output polygon
            #
            rad = math.radians(angle)
            rad2 = math.radians(-rotation)
            for i in range(0, sidesCnt):
                outPoint.x = inPoint.x - math.sin(rad * i + rad2) * distance
                outPoint.y = inPoint.y + math.cos(rad * i + rad2) * distance
                lineArray.Add(outPoint)

            # Create the new output feature and insert
            #
            feat = iCur.NewRow()
            feat.SetValue(shapeName, lineArray)
            for field in fieldList:
                feat.SetValue(field, sRow.GetValue(field))
            iCur.InsertRow(feat)

            lineArray.RemoveAll()
            sRow = sCur.Next()            
        
    except Exception, err:
        print err.message
        gp.AddError(err.message)

    finally:    
        if iCur:
            del iCur
        if sRow:
            del sRow
        if sCur:
            del sCur
        if feat:
            del feat

if __name__ == '__main__':
    runRegularPolygons()    


